package pintores;

import java.util.Random;

import jade.core.Agent;
import jade.core.behaviours.CyclicBehaviour;
import jade.core.behaviours.OneShotBehaviour;
import jade.core.behaviours.SequentialBehaviour;
import jade.core.behaviours.SimpleBehaviour;
import jade.core.behaviours.TickerBehaviour;
import jade.domain.DFService;
import jade.domain.FIPAException;
import jade.domain.FIPAAgentManagement.DFAgentDescription;
import jade.domain.FIPAAgentManagement.ServiceDescription;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.UnreadableException;

public class PantallaAgent extends Agent {
	private Pantalla pantalla;

	protected void setup() {
		
		SequentialBehaviour startUp = new SequentialBehaviour(this) {
			@Override
			public int onEnd() {
				System.out.println("Pantalla ready and registered");
				return super.onEnd();
			}
		};
		
		startUp.addSubBehaviour(new OneShotBehaviour(this) {
			
			@Override
			public void action() {
				pantalla = new Pantalla();
				pantalla.setVisible(true);
				System.out.println("Pantalla visible");
			}
		});
		
		startUp.addSubBehaviour(new OneShotBehaviour(this) {
			
			@Override
			public void action() {
				DFAgentDescription description = new DFAgentDescription();
				description.setName(getAID());
				System.out.println("Pantalla AID to register: " + getAID());
				ServiceDescription serviceDescription = new ServiceDescription();
				serviceDescription.setType("pantalla");
				serviceDescription.setName("panTAIA");
				description.addServices(serviceDescription);
				try {
					DFService.register(this.getAgent(), description);
					System.out.println("Pantalla successfully registered.");
				} catch (FIPAException fe) {
					fe.printStackTrace();
				}
			}
		});

		
		
		
		addBehaviour(startUp);
		addBehaviour(new WaitForMessageAndPaintBehaviour(this));
		addBehaviour(new TickerBehaviour(this, 100) {
			
			@Override
			protected void onTick() {
				((PantallaAgent) this.myAgent).repaint();
				//System.out.println("Pantalla: repainting...");
			}
		});
	}
	
	

	class WaitForMessageAndPaintBehaviour extends CyclicBehaviour {
		public WaitForMessageAndPaintBehaviour(Agent a) {
			super(a);
		}

		public void action() {
			ACLMessage msg = receive();
			if (msg != null) {
				if (msg.getPerformative() == ACLMessage.REQUEST) {
					try {
						DrawableObject d = (DrawableObject) msg.getContentObject();
						pantalla.objects.add(d);
					} catch (UnreadableException e) {
						e.printStackTrace();
					}
				}
			}
			System.out.println("Pantalla: Message recieved.");
			this.block();
		}
	}
	
	public void repaint() {
		pantalla.repaint();
	}
}
