package pintores;

import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Random;

import jade.content.onto.basic.Done;
import jade.core.AID;
import jade.core.Agent;
import jade.core.behaviours.Behaviour;
import jade.core.behaviours.CyclicBehaviour;
import jade.core.behaviours.SequentialBehaviour;
import jade.core.behaviours.TickerBehaviour;
import jade.core.behaviours.WakerBehaviour;
import jade.domain.DFService;
import jade.domain.FIPAException;
import jade.domain.FIPAAgentManagement.DFAgentDescription;
import jade.domain.FIPAAgentManagement.ServiceDescription;
import jade.lang.acl.ACLMessage;

public class PintorAgent extends Agent {
	int lifeTime = 20;
	String speciality = "line";
	AID[] pantallaAID = null;

	protected void setup() {

		parseArguments();

		final TickerBehaviour paintSomething = new TickerBehaviour(this, 400) {
			private boolean finished = false;

			@Override
			public void onStart() {
				((PintorAgent) this.myAgent).say("I'm planing to paint "
						+ lifeTime + " " + speciality + "s onto the board.");
				super.onStart();
			}

			protected void onTick() {
				lookForPantallas();
				DrawableObject d = new Line();
				if (speciality.equals("circle")) {
					d = new Circle();
				}
				if (speciality.equals("rectangle")) {
					d = new Rectangle();
				}
				sendDrawableObject(d);
				if (this.getTickCount() >= lifeTime) {
					stop();
				}
			}

			@Override
			public int onEnd() {
				say("That's enough of those " + speciality + "s.");
				this.getAgent().doDelete();
				return super.onEnd();
			}
		};
		
		

		WakerBehaviour startBreak = new WakerBehaviour(this, 10000) {
			protected void onWake() {
				say("I'm having a break now...");
				removeBehaviour(paintSomething);
			}
		};

		WakerBehaviour endBreak = new WakerBehaviour(this, 13000) {
			protected void onWake() {
				say("Back to work!");
				addBehaviour(paintSomething);
			}
		};
		
		WakerBehaviour paintHouse = new WakerBehaviour(this, 14000) {
			protected void onWake() {
				say("Now let's paint a house...");
				addBehaviour(new PaintHouseInThreeStepsBehaviour());
			}
		};

		addBehaviour(paintHouse);
		addBehaviour(paintSomething);
		addBehaviour(startBreak);
		addBehaviour(endBreak);
	}

	private void parseArguments() {
		Object[] args = getArguments();
		if (args != null && args.length > 0) {
			speciality = (String) args[0];
		}
		if (args != null && args.length > 1) {
			try {
				lifeTime = Integer.parseInt((String) args[1]);
			} catch (NumberFormatException e) {
				System.err.println("Argument 1 must be an integer");
			}
		}
	}

	@Override
	protected void takeDown() {
		say("Enough work for today, I go home now.");
	}

	private void say(String s) {
		System.out.println(this.getLocalName() + ": " + s);
	}

	private void lookForPantallas() {
		DFAgentDescription agentDescription = new DFAgentDescription();
		ServiceDescription serviceDescription = new ServiceDescription();

		serviceDescription.setType("pantalla");
		agentDescription.addServices(serviceDescription);

		try {
			DFAgentDescription[] result = DFService.search(this,
					agentDescription);
			pantallaAID = new AID[result.length];
			for (int i = 0; i < result.length; ++i) {
				pantallaAID[i] = result[i].getName();
			}
		} catch (FIPAException fe) {
			fe.printStackTrace();
		}
	}

	private void sendDrawableObject(DrawableObject drawable) {

		if (pantallaAID.length >= 1) {
			AID r = pantallaAID[0];
			ACLMessage msg = new ACLMessage(ACLMessage.REQUEST);
			msg.setSender(getAID());
			msg.addReceiver(r);
			try {
				msg.setContentObject(drawable);
			} catch (IOException e) {
				e.printStackTrace();
			}
			say("I paint a " + drawable.getName() + " onto the board.");
			send(msg);
		} else {
			say("No board found yet.");
		}
	}
	
	private class PaintHouseInThreeStepsBehaviour extends Behaviour {
		
		private int step = 1;
		
		@Override
		public void action() {
			switch (step) {
			case 1:
				say("Painting the main part of the house...");
				sendDrawableObject(new Rectangle(100,100,40,40));
				break;
			case 2:
				say("Painting the door...");
				sendDrawableObject(new Rectangle(120,120,10,20));
				break;
			case 3:
				say("And finally the roof.");
				sendDrawableObject(new Line(100,100,120,80));
				sendDrawableObject(new Line(140,100,120,80));
				break;
			default:
				break;
			}
			step++;
		}

		@Override
		public boolean done() {
			return step > 3;
		}
	}
}
